/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardApplication");

dojo.require("net.jazz.ajax.ui._Application");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.ui.wizard.Wizard");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardIntro");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardInfo");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardMigration");

dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardApplicationMessages");

dojo.require("com.ibm.rdm.fronting.server.web.util.domutils");
dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");

dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.jfs.RRCOAuthInterceptor");
dojo.require("com.ibm.rdm.fronting.server.web.ui.internal.jfs.RRCOAuthDialog");

function workbenchRootNode(){
	return net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode();
}

var domutils = com.ibm.rdm.fronting.server.web.util.domutils;
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;
var Wizard = com.ibm.team.repository.web.ui.wizard.Wizard;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var Intro = com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardIntro;
var Info = com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardInfo;
var Migration = com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardMigration;

jazz.client.internal.addResponseInterceptor(com.ibm.rdm.fronting.server.web.ui.internal.jfs.RRCOAuthInterceptor); 

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardApplication", net.jazz.ajax.ui._Application, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardApplicationMessages");
		this._startApp(); //new
	},	

	_startApp: function () {
		this._createUI();
		this.setName(this.messages.setupName);
		this.start();
	},
	
	_createUI: function() {		
		dojo.addClass(workbenchRootNode(), "com-ibm-team-repository-web-setup-RootNode");
		this.content = document.createElement("div");
		dojo.addClass(this.content, "com-ibm-team-repository-wizard");
		
		var textHeader = document.createElement("div");
		textHeader.innerHTML= this.messages.setupHeading;
		textHeader.className = "setup-header";
		this.content.appendChild(textHeader);
		
		var jazzIcon = document.createElement("img");
		jazzIcon.src = dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/images/jazz-identR.png");
		jazzIcon.className = "jazz-icon";
		this.content.appendChild(jazzIcon);
		
		var setupGraphic = document.createElement("img");
		setupGraphic.src = dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/images/JTS-wizard-graphic.png");
		setupGraphic.className = "wizard-graphic";
		this.content.appendChild(setupGraphic);

		this.wizardBox = document.createElement("div");
		this.content.appendChild(this.wizardBox);
				
		var wizardParams = {
			fullPageContainer: this.wizardBox,
			processFinish:  function(){
				//get rootservices and parse for JFS Url
				var rrsUrl = rrsutils.getRRSUrl(document.location.href);
				var webUIRoot = "" + rrsutils.getRootServiceUrl(rrsUrl, "viewletServiceRoot", domutils.JD_NAMESPACE, domutils.JD_QUALIFIER);

				document.location.href = webUIRoot + "/admin";
            }
		};
		
		wizardParams.steps = this.initServerSetupSteps();
		
		/*
		if (this.authorized) { 
			wizardParams.steps = this.initServerSetupSteps();
			
		} else {
			var notAuthStep = new NotAuthorizedView({applicationName: this.messages.setupName});
			dojo.mixin(notAuthStep, new WizardStep());
			notAuthStep.includeNavigationUI = false;
			wizardParams.steps = [notAuthStep];
			wizardParams.showStepSummary = false;
		}
		*/
		var wizard = new Wizard(wizardParams);
		
		var outerFooter = new com.ibm.rdm.fronting.server.web.ui.internal.wizard.LogoFooter();
		this.content.appendChild(outerFooter.domNode);
		
		workbenchRootNode().appendChild(this.content);
		
	},
	
	initServerSetupSteps: function () {
		var steps = [
			new Intro(),
			new Info(),
			new Migration()
		];
		return steps;
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function(errorObject) {
		this._startApp();
	},
	
	invokeServer: function () {
		// To be invoked asynchronously to activate the services on a newly started server
		var srh= new ServiceResponseHandler(this, "doNothing", "doNothing");
		ServerStatusClient.getServiceConfiguration(srh, "com.ibm.team.repository.service.internal.MailerService");
	}, 
	
	doNothing: function () {
		// do nothing
	}
});
var messages;
dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.LogoFooter", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/templates/LogoFooter.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardApplicationMessages");
	}
});

})();