/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.app.auth");
dojo.require("jazz.ui.Button");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.util.domutils");
dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");

(function() {

var domutils = com.ibm.rdm.fronting.server.web.util.domutils;
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;
var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var CONFIGURATION_VERSION = "3";
var canProceed = false;
var rrsUrl;
var timerId;
var timerRebuildId;

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardInfo", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/templates/Configure.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepConfigureTitle;
	},
	
	postCreate: function(){
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.rrsUrl = rrsutils.getRRSUrl(document.location.href);
		this.processParams();
	},
	
	focusStep: function () {
		this.checkConfigurationVersion();
	},
		
	canFlipToNextPage: function() {
		return canProceed;
	},
	
	canFinish: function() {
		return this.canFlipToNextPage();
	},
	
	processParams: function(){	
		var qstr = window.location.search.substr(1);
		
		//update the fronting server OAuth consumer key and secret ASAP
		if (qstr.indexOf("k=") > -1 && qstr.indexOf("s=") > -1) {	
			//build the params
			var params = "?" + qstr;

			//post to JFS
			this.postOAuth(this.rrsUrl + "/configure" + params);
		}
	},
			
	checkConfigurationVersion: function() {
		var self = this;

		var result = jazz.client.xhrGet({
			url: this.rrsUrl + "/project-resources/configure",
			sync: true,
			handleAs: "text",
			headers: {
		    	"Accept": "*"
		    },
			load: function(data, ioargs){
				if (data == CONFIGURATION_VERSION) {
					self._rebuildButton.disabled = false;
					canProceed = true;
					self.triggerButtonStateUpdate();
				} else {
					self._rebuildButton.disabled = true;
					canProceed = false;
					self.triggerButtonStateUpdate();
				}
			},
			error: function(error, ioargs){
                switch (ioargs.xhr.status) {
                case 404:
                case 410:
                    self._configureButton.disabled = false;
                    console.log("Rebuild will only be enabled when the server is configured");
                    break;
                case 401:
                case 403:
                	self._configureButton.disabled = true;
                    self._showConnectionStatus("ERROR", messages.Configuration_Authentication_Failed);
                    break;
                default:
                   console.log(error);
                }

				self._rebuildButton.disabled = true;
				canProceed = false;
				self.triggerButtonStateUpdate();
			}	        
		});
	},

	postOAuth: function(rrsUrl) {	
		var result = jazz.client.xhrPost({
	  		url: rrsUrl,
	  		sync: true,
	  		load: function(data, ioargs){
	  			//TODO check for 201
	        },
	        error: function(error){
	        }	        
	  	});
	},
	
	onConfigureButtonClick: function() {
		//do a POST to /configure service with OAuth consumer key and secret
		var self = this;
 
		var result = jazz.client.xhrPost({
	  		url: self.rrsUrl + "/configure?setup=true",
	  		load: function(data, ioargs){
	  			//start the status check
	  		  	timerId = setInterval(function() { self.getStatus(); }, 500);
	        },
	        error: function(error, ioargs){
	        	if (ioargs.xhr.status === 403) {
	        		self._showConnectionStatus("ERROR", error);
	        	} else {
	          		self._showConnectionStatus("ERROR", messages.Configuration_Setup_Failed, error);
	          	}
	        }	        
	  	});	
	},
	
	getStatus: function() {	
		var self = this;
		
		//check the status of the configuration
  		jazz.client.xhrGet({
			url: self.rrsUrl + "/configure/status",
			headers: {
                "Accept": "application/json"
            },
			load: function(data, ioargs){
				var json = eval('(' + data + ')');
				var details = parseData(json);
				
				if (json.complete === true) {
					//stop the status check
					clearInterval(timerId);
					
					var status;
					var message;
					if(typeof(json.errors) != "undefined") {
						status = "ERROR";
						message = messages.Configuration_Setup_Failed;
						self._configureButton.disabled = false;
						canProceed = false;
						self.triggerButtonStateUpdate();
					} else {
						status = "OK";
						message = messages.Configuration_Setup_Completed;
						self._configureButton.disabled = true;
						canProceed = true;
						self.triggerButtonStateUpdate();
					}
		
					self._showConnectionStatus(status, message, details);
				} else {
					self._showConnectionStatus("PROGRESS", messages.Configuration_Running, details);
				}
			},
			error: function(error){
				self._showConnectionStatus("ERROR", error);
			}	        
		});
	},
	
	onRebuildButtonClick: function() {
		//do a POST to /configure service with ?rebuild=true
		var self = this;
		jazz.client.xhrPost({
	  		url: self.rrsUrl + "/configure?rebuild=true",
	  		load: function(data, ioargs){	  		
	  			//start the status check
	  		  	timerRebuildId = setInterval(function() { self.getRebuildStatus(); }, 500);
	        },
	        error: function(error){
	          	self._showConnectionStatus("ERROR", error);
	        }	        
	  	});	
	},

	parseRebuildData: function(data) {
		var time = 0;
		
		//parse response				
		var nodes = domutils.getElementsByTagNameNS(
				 data, 
				 "remainingTime",
				 domutils.JAZZ_NAMESPACE, 
				 domutils.JAZZ_QUALIFIER);
 
		for (var i=0; i<nodes.length; i++) {
			var node = nodes[i];
			var remainingTime = (node.firstChild ? node.firstChild.nodeValue : "0");
			time = time + parseInt(remainingTime);
		}	
		
		//in seconds
		return Math.round(time/1000);
	},
	
	getRebuildStatus: function() {	
		var self = this;
		var response;
		//check the status of the configuration
  		var result = jazz.client.xhrGet({
			url: self.rrsUrl + "/configure/indexing",
			//sync: true,
			handleAs: "xml",
			headers: {
		    	"Accept": "application/rdf+xml"
		    },
			load: function(data, ioargs){
				//parse response	
				var time = self.parseRebuildData(data);
							
				if (time === 0) {
					clearInterval(timerRebuildId);
					self._showConnectionStatus("OK", messages.Configuration_Rebuild_Completed);
				} else {
					self._showConnectionStatus("PROGRESS", messages.Configuration_Rebuild_Running, dojo.string.substitute(messages.Configuration_Rebuild_Progress, [time]));
				}
			},
			error: function(error){
				self._showConnectionStatus("ERROR", error);
			}	        
		});
		return response;
	},
	
	_showConnectionStatus: function(type, summary, details) {
		if (type != "NONE" && type != "PROGRESS") {
			var headerSummary = summary;
			if (type == "OK") {
				headerSummary = headerSummary + "  " + messages.clickNext;	
			}
		
			var detailsMessagesArray;
			if(typeof(details) != "undefined") { 
				var detailsArray = details.split("<br>");
				detailsMessagesArray = new Array();
	
				//Get the actual details message
				for (var i = 0; i < detailsArray.length; i++) { 
					var detailsMessage = {};
					detailsMessage.message = detailsArray[i];
					detailsMessagesArray[i] = detailsMessage;
				}
			}
			
			this.header.showStatus({type: type,	summary: headerSummary, details: detailsMessagesArray});
		} else {
			this.header.clearMessages();
		}
		this._setStatusIndication(type, summary, details);
	}, 
	
	_setStatusIndication: function (type, summary, details) {
		
		// append summary to details
		var buf = "";
		buf = buf + "<h3>";
		buf = buf + summary;
		buf = buf + "</h3>";
		buf = buf + "<br>";
		if(typeof(details) != "undefined") {
			buf = buf + details;
		}
		summary = buf;
		
		if (type != "INFO" && type != "NONE") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (type != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}	
			
		} else {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		}
	}		
		
});

function parseData (json) {
	var buf = "";
	
	//errors
	if(typeof(json.errors) != "undefined") {
		for (var i = 0; i < json.errors.length; i++) {
			buf = buf + json.errors[i];
			buf = buf + "<br>";
		}
		buf = buf + "<br>";
	}
	
	//results
	for (var i = 0; i < json.results.length; i++) {
		buf = buf + json.results[i];
		buf = buf + "<br>";
	}
	
	return buf;
}

})();