/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardIntro");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.util.domutils");
dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");

(function() {

var domutils = com.ibm.rdm.fronting.server.web.util.domutils;
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;
var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var newConsumer = true;
var isAuthorized = false;

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardIntro", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/templates/Intro.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepIntroTitle;
	},
	
	postCreate: function() {
		var header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(header.domNode);
		
		//get base Url
		var curPage = new dojo._Url(document.location.href);
		var path = curPage.path.toString();
		var rrsUrl = curPage.toString().substring(0, curPage.toString().indexOf(path));

		//set JFS Url
		var serverRootUrl = new dojo._Url(net.jazz.ajax._dataServerRoot);
		var rootPath = serverRootUrl.path.toString();
		if (rootPath.length <= 0) {
			rootPath = "/jazz"; //in case _dataServerRoot is not defined with a path
		}
		this.JFSurl.value = rrsUrl + rootPath;
		
		//set RRS Url
		this.frontingHostName.value = rrsUrl + net.jazz.ajax._contextRoot;
		
		this.setVersion();
		
		this.checkOAuth();
		
		//handle form validation
		this.dataFields = [];
		this.dataFields.push(this.JFSurl);
		this.dataFields.push(this.frontingHostName);
		this.validate();
	},
	
	setVersion: function() {
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		
		var version = "" + rrsutils.getRootServiceUrl(rrsUrl, "buildVersion", domutils.OSLC_RM_NAMESPACE, domutils.OSLC_RM_QUALIFIER);
		
		this.RRSVersion.innerHTML = messages.RRS + version;
	},
	
	// Check to force the user to login if OAuth key has been set.  Must be synchronous.
	checkOAuth: function() {
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
		var self = this;
		
		jazz.client.xhrGet({
			url: rrsUrl + "/configure/oauth",
			sync: true,
			load: function(data, ioargs){
				if (200 == ioargs.xhr.status) {
		            self.isAuthorized = true;
		            self.triggerButtonStateUpdate();
		        }
			}	        
		});
	},
	
	//form events
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	//form events
	handleChange: function(e) {
		this.validate();
	},
	
	//check for empty fields
	validate: function() {
	
		var isValid = true;
		if (this.dataFields) {
			for (var i = 0; i < this.dataFields.length; i++) {
				if (isValid) {
					var input = this.dataFields[i];
					isValid = rrsutils.validateURLHostField(input.value);
				}
			}
		}
	
		clearTimeout(this.onKeyUpTimout);
		
		var formStatus = {
			isComplete: isValid
		};
		this.setStatus(formStatus);
		return formStatus;
	},
	
	// invalid data disables the next button
	setStatus: function(status) {
		var canClickNext = false;
		if (status.isComplete) {
			canClickNext = true;
		}
		this.enableNext = canClickNext;
		this.triggerButtonStateUpdate();
	},
	
	// WIZARD STEP
	canFlipToNextPage: function () {
		return (this.enableNext === true && this.isAuthorized === true);
	},
	
	onNext: function() {
		
		var rrsUrl = rrsutils.getRRSUrl(document.location.href);
	
		var jfsUrl = this.JFSurl.value;

		//post the properties to the fronting server
		if (jfsUrl.length > 0) {
			if (jfsUrl.charAt(jfsUrl.length-1) == "/") {
				jfsUrl = jfsUrl.substring(0, jfsUrl.length-1);
			}

			var frontingUrl = new dojo._Url(this.frontingHostName.value);

			this.postJFSUrl(
				rrsUrl + "/configure" +
				"?jfsUrl=" + encodeURIComponent(this.JFSurl.value) +
				"&hostName=" + encodeURIComponent(frontingUrl.host));
		}
		
		var ref = window.location + "";
		
		//Strip out the current step information
		ref = ref.substring(0,ref.indexOf('#')); 
		
		//Append "next step" information
		ref += "#/steps/2";
		
		//Check for a trailing / before appending /setup
		var url = jfsUrl;
		url += (url.charAt(url.length-1) == "/") ? 'setup' : '/setup';

		//GET /rdm/friends to see if the OAuth credentials have been defined	
		var hasOAuth = this.getHasOAuth(rrsUrl);
		
		//Do the redirect to the JFS setup wizard & append query params
		if (!hasOAuth) {
			//Provide the name of our OAuth consumer to associate it with the consumer key
			var oauthConsumerName = "Rational Requirements Server";
			var oauthId = "rrs-daemon";
			var rootservices = this.buildRootservices(rrsUrl);
			
			url = url +
				'?oauth=' + encodeURIComponent(oauthConsumerName) +
				'&oauthId=' + encodeURIComponent(oauthId) +
				'&rootservices=' + encodeURIComponent(rootservices) +
				'&ref=' + encodeURIComponent(ref);
		} else {
			url = url + '?ref=' + encodeURIComponent(ref);
		}

		window.location = url;
	},
	
	postJFSUrl: function(rrsUrl) {	
		var result = jazz.client.xhrPost({
	  		url: rrsUrl,
	  		sync: true,
	  		load: function(data, ioargs){
	  			//TODO check for 201
	        },
	        error: function(error){
	        }	        
	  	});
	},

	buildRootservices: function(rrsUrl) {	
		var rootservices = new dojo._Url(rrsUrl);
		rootservices.host = this.frontingHostName.value;
		return rootservices.toString() + "/rootservices";
	},

	getHasOAuth: function(rrsUrl) {	
		var ret = false;
		var result = jazz.client.xhrGet({
			url: rrsUrl + "/friends",
			sync: true,
			handleAs: "xml",
			headers: {
		    	"Accept": "application/rdf+xml"
		    },
			load: function(data, ioargs){
				//parse response				
				var artifactNodes = domutils.getElementsByTagNameNS(
						 data, 
						 "Friend",
						 domutils.JD_NAMESPACE, 
						 domutils.JD_QUALIFIER);
		 
				for (var i=0; i<artifactNodes.length; i++) {
					var titleNode = domutils.getElementsByTagNameNS(
						artifactNodes[i], 
						"title", 
						domutils.DC_NAMESPACE, 
						domutils.DC_QUALIFIER)[0];
						
	    			var title = (titleNode.firstChild ? titleNode.firstChild.nodeValue : "");

					if (title == "RRS_JFS") {
						ret = true;
					}
				}
			},
			error: function(error){
				console.log(error);
			}	        
		});
		return ret;
	}

});

})();