/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardMigration");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStep");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");

dojo.require("com.ibm.rdm.fronting.server.web.util.rrsutils");

(function() {
var rrsutils = com.ibm.rdm.fronting.server.web.util.rrsutils;
var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
var finished = false;
var timeout;
var rrsUrl;
var header;

dojo.declare("com.ibm.rdm.fronting.server.web.ui.internal.wizard.WizardMigration", [WizardStep, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.rdm.fronting.server.web", "ui/internal/wizard/templates/Migration.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.rdm.fronting.server.web.ui.internal.wizard", "WizardMessages");
		this.stepTitle = messages.stepMigrationTitle;
	},
	
	postCreate: function(){
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
	},

	canFinish: function() {
		return true;
	},
	
	canFlipToNextPage: function() {
		return false;
	},
	
	onSelectOption: function(e) {
		if(!e.target || e.target.tagName !== "INPUT") {return;}
		
		var selected = this.getSelectedOption();
		this.displayMigration(selected);
	},
	
	getSelectedOption: function() {
		var inputs = this.migrationOptions.getElementsByTagName("input");
		var value = null;
		for (var i=0; i<inputs.length; i++) {
			if (inputs[i].checked) {
				value = inputs[i].value;
				break;
			}
		}
		return value;
	},
	
	displayMigration: function(selected) {
		var disabled = true;
		if (selected === "YES") {
			disabled = false;
		}
		this.jrsUrl.disabled = disabled;
		this.jrsUser.disabled = disabled;
		this.jrsPassword.disabled = disabled;
		this.functional.disabled = disabled;
		this._migrateButton.disabled = disabled;
	},

	onMigrateButtonClick: function() {
		//do a POST to /migration service with query params
		this.rrsUrl = rrsutils.getRRSUrl(document.location.href);
		
		this.postMigration(rrsUrl);
	},
	
	postMigration: function(rrsUrl) {			
		var jrsUrl = this.jrsUrl.value;
		var jrsUser = this.jrsUser.value;
		var jrsPassword = this.jrsPassword.value;
		var functional = this.functional.checked;
		var rrsUrl = this.rrsUrl;
		
		var params = "?jrsURL=" + encodeURIComponent(jrsUrl) + "&username=" + encodeURIComponent(jrsUser) + "&password=" + encodeURIComponent(jrsPassword) + "&functional=" + encodeURIComponent(functional);
		
		var self = this;
			
		var doStatusFunction = dojo.hitch(this, "doStatus");
		
		var result = jazz.client.xhrPost({
	  		url: this.rrsUrl + "/migration" + params,
	  		load: function(data, ioargs){
				self.timeout = setInterval(doStatusFunction, 1000);
	        },
	        error: function(error){
	          	self._showConnectionStatus("ERROR", messages.Migration_Failed, error);
				clearInterval(self.timeout);
	        }	        
	  	});	
	},
		
	getStatus: function(rrsUrl) {	
		var self = this;
		var response;
		//check the status of the configuration
  		var result = jazz.client.xhrGet({
			url: rrsUrl + "/migration?status",
			sync: true,
			headers: {
                "Accept": "application/json"
            },
			load: function(data, ioargs){
				response = data;
				return data;
			},
			error: function(error){
				self._showConnectionStatus("ERROR", messages.Migration_Failed, error);
			}	        
		});
		return response;
	},
	
	_showConnectionStatus: function(type, summary, details) {
		if (type != "NONE" && type != "PROGRESS") {
			var headerSummary = summary;
			if (type == "OK") {
				headerSummary = headerSummary;	
			}
		
			var detailsMessagesArray;
			if(typeof(details) != "undefined") { 
				var detailsArray = details.split("<br>");
				detailsMessagesArray = new Array();
	
				//Get the actual details message
				for (var i = 0; i < detailsArray.length; i++) { 
					var detailsMessage = {};
					detailsMessage.message = detailsArray[i];
					detailsMessagesArray[i] = detailsMessage;
				}
			}
			
			this.header.showStatus({type: type, summary: headerSummary, details: detailsMessagesArray})
		} else {
			this.header.clearMessages();
		}
		this._setStatusIndication(type, summary, details);
	}, 
	
	_setStatusIndication: function (type, summary, details) {
		
		// append summary to details
		var buf = "";
		buf = buf + "<h3>";
		buf = buf + summary
		buf = buf + "</h3>";
		buf = buf + "<br>";
		if(typeof(details) != "undefined") {
			buf = buf + details;
		}
		summary = buf;
		
		

		if (type != "INFO" && type != "NONE") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + type;
			indication.innerHTML = summary;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (type != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}	
			
		} else {
			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
		}
	},

	doStatus: function () {
		var doStatusSelf = this;
		var response = doStatusSelf.getStatus(doStatusSelf.rrsUrl);
		var json = eval('(' + response + ')');
		var progress = parseData(json);
		
		if (json.remainingTasks == 0) {
			doStatusSelf.finished = true;
			clearInterval(doStatusSelf.timeout);
			if(typeof(json.errors) != "undefined") {
				doStatusSelf._showConnectionStatus("ERROR", messages.Migration_Failed, progress);
			}
			else{
				doStatusSelf._showConnectionStatus("OK", messages.Migration_Completed, progress);
			}
			} else {
					doStatusSelf._showConnectionStatus("PROGRESS", messages.Migration_Running, progress);
			}	
	}
		
});

function parseData (json) {

	var buf = "";

	// heading
	buf = buf + "<h3>";

	buf = buf + "</h3>";
	
	buf = buf + "<br>";
	
	//errors
	if(typeof(json.errors) != "undefined") {
		for (var i = 0; i < json.errors.length; i++) {
			buf = buf + "<font color=\"red\">"+json.errors[i]+"</font>";
			buf = buf + "<br>";
		}
		buf = buf + "<br>";
	}
	
	//results
	for (var i = 0; i < json.results.length; i++) {
		buf = buf + json.results[i];
		buf = buf + "<br>";
	}
	
	return buf;//"Hi<br>";
}

})();